package main;

import java.awt.Dialog.ModalityType;

import java.util.List;
import java.util.Observable;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.EntityTransaction;
import javax.persistence.Persistence;
import javax.persistence.Query;
import javax.persistence.TypedQuery;
import javax.swing.SwingUtilities;
import javax.swing.table.DefaultTableModel;

import view.FahrerFrame;
import view.MainFrame;

import model.databaseModel.*;

/**
 * 
 * Controller-Klasse des MVC-Patterns Einfhrung in den SQL-Dialekt JPQL von JPA
 * siehe hier: http://www.objectdb.com/java/jpa/query
 * 
 * @author Thomas Kramer
 * 
 */
public class Controller extends Observable {
	private MainFrame viewMF = null;
	private FahrerFrame viewFF = null;

	private Controller controller = null;
	public static EntityManagerFactory emf = Persistence
			.createEntityManagerFactory("JPA");
	public static EntityManager em = emf.createEntityManager();

	/**
	 * parameterloser Konstruktor
	 */
	public Controller() {
		controller = this;

		/**
		 * erzeuge alle Fenster und zeige Hauptfenster an.
		 */
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				/**
				 * Fenster erzeugen und dem Hauptfenster verfgbar machen
				 */
				viewMF = new MainFrame();
				viewMF.setSize(800, 300);
				viewMF.setVisible(true);
				// positionieren des Fensters: mittig
				viewMF.setLocationRelativeTo(null);

				viewFF = new FahrerFrame(viewMF);
				viewFF.setModalityType(ModalityType.DOCUMENT_MODAL);
				viewFF.setVisible(false);

				viewMF.viewFF = viewFF;

				/**
				 * Beobachter hinzufgen
				 */
				controller.addObserver(viewMF);
				controller.addObserver(viewFF);

				/**
				 * Daten-Neuladen fr Views veranlassen.
				 */
				tellChangedData();

				// viewMF.pack();
				viewMF.setVisible(true);
			}
		});
	}

	/**
	 * 
	 * Auslesen der Fahrer-Daten und Zurckliefern ber eine neue Liste
	 * 
	 */
	public static List<Fahrer> selectFahrer() {
		List<Fahrer> results = null;
		try {
			TypedQuery<Fahrer> query = em.createQuery("SELECT C FROM Fahrer C",
					Fahrer.class);
			results = query.getResultList();
		} catch (Exception e) {			
			e.printStackTrace(System.out);			
		}
		return results;
	}

	/**
	 * 
	 * Auslesen der Fahrzeug-Daten und Zurckliefern ber eine neue Liste
	 * 
	 */
	public static List<Fahrzeug> selectFahrzeuge() {
		List<Fahrzeug> results = null;
		try {
			TypedQuery<Fahrzeug> query = em.createQuery(
					"SELECT C FROM Fahrzeug C", Fahrzeug.class);
			results = query.getResultList();
		} catch (Exception e) {
			e.printStackTrace(System.out);			
		}
		return results;
	}

	/**
	 * 
	 * Auslesen der Fahrer-Fahrzeug-Zuordnungen und Zurckliefern ber eine neue
	 * Liste
	 * 
	 */
	public static List<Zustaendigkeit> selectZustaendigkeit() {
		List<Zustaendigkeit> results = null;
		try {
			TypedQuery<Zustaendigkeit> query = em.createQuery(
					"SELECT C FROM Zustaendigkeit C", Zustaendigkeit.class);
			results = query.getResultList();
		} catch (Exception e) {
			e.printStackTrace(System.out);			
		}
		return results;
	}

	/**
	 * 
	 * Auslesen eines bestimmten Fahrers aus einer Zuordnung
	 * 
	 */
	public static Fahrer getFahrer(Integer Fahrer_ID) {
		Fahrer result = null;
		try {
			Query query = em
					.createQuery("SELECT C FROM Fahrer C WHERE C.fahrerid=:F_ID");
			query.setParameter("F_ID", Fahrer_ID);

			result = ((Fahrer) query.getSingleResult());
		} catch (Exception e) {
			e.printStackTrace(System.out);
		}
		return result;
	}

	/**
	 * 
	 * Auslesen eines bestimmten Fahrzeugs aus einer Zuordnung
	 * 
	 */
	public static Fahrzeug getFahrzeug(Integer Fahrzeug_ID) {
		Fahrzeug result = null;
		try {
			Query query = em
					.createQuery("SELECT C FROM Fahrzeug C WHERE C.fahrzeugid=:F_ID");
			query.setParameter("F_ID", Fahrzeug_ID);

			result = ((Fahrzeug) query.getSingleResult());
		} catch (Exception e) {
			e.printStackTrace(System.out);
		}
		return result;
	}

	/**
	 * Einfgen einer neuen Fahrer/Fahrzeug-Zuordnung
	 * 
	 * @return boolean - ob das Einfgen in DB funktioniert hat oder nicht
	 */
	public static boolean insertFahrerAndZustaendigkeit(String name,
			String fuehrerscheinklasse, String fuehrerscheinSeit,
			Fahrzeug fahrzeug1, Fahrzeug fahrzeug2, Fahrzeug fahrzeug3) {
		boolean result = false;		
		if (fahrzeug1 == null)
		{
			return result;
		}
		
		EntityManagerFactory localEMF = Persistence
				.createEntityManagerFactory("JPA");
		EntityManager localEM = localEMF.createEntityManager();

		EntityTransaction transaction = null;
		try {
			transaction = localEM.getTransaction();
			transaction.begin();

			/**
			 * Neue Datenstze anlegen
			 */
			Fahrer fahrer = new Fahrer();
			fahrer.setName(name);
			fahrer.setFuehrerscheinklasse(fuehrerscheinklasse);
			fahrer.setFuehrerscheinseit(fuehrerscheinSeit);

			// Datensatz schreiben
			// http://www.developerfusion.com/article/84945/flush-and-clear-or-mapping-antipatterns/
			localEM.persist(fahrer);

			// (nach schreibenden Zugriffen) alle nderungen werden mit der DB
			// synchronisiert
			// und beim Commit der TX in die DB geschrieben
			// siehe hier:
			// http://www.prozesse-und-systeme.de/jpaEinleitung.html
			localEM.flush();

			// Daten endgltig bermitteln, flush() muss in der Transaktion
			// ablaufen
			// beim commit wird die ID des Datensatzes ermittelt
			transaction.commit();

			// Entity aktualisieren, um aktuelle ID zu erhalten
			localEM.refresh(fahrer);

			////////////////////////////////////////////////////////////////
			// Assoziations-Datensatz schreiben
			////////////////////////////////////////////////////////////////			

			transaction.begin();

			Zustaendigkeit zustaendigkeit = new Zustaendigkeit();
			zustaendigkeit.setFahrerid(fahrer.getFahrerid());
			zustaendigkeit.setFahrzeug1id(fahrzeug1.getFahrzeugid());
			if (fahrzeug2 != null) {
				zustaendigkeit.setFahrzeug2id(fahrzeug2.getFahrzeugid());
			}
			if (fahrzeug3 != null) {
				zustaendigkeit.setFahrzeug3id(fahrzeug3.getFahrzeugid());
			}

			localEM.persist(zustaendigkeit);			
			localEM.flush();			

			/**
			 * Daten bermitteln
			 */
			transaction.commit();

			result = true;
		} catch (Exception e) {
			if (transaction != null && transaction.isActive()) {
				transaction.rollback();
			}
			result = false;
		} finally {
			localEM.close();
			localEMF.close();
		}
		return result;
	}

	/**
	 * 
	 * benachrichtige Views, dass Daten gendert wurden.
	 * 
	 */
	public void tellChangedData() {
		// setChanged() markiert das Objekt als sendebereit, sodass bei
		// notifyObservers() Meldungen gegeben werden.
		setChanged();
		notifyObservers();
	}
}
