package view;

import java.awt.HeadlessException;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.text.ParseException;
import java.util.List;
import java.util.Observable;
import java.util.Observer;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JFormattedTextField;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;
import javax.swing.text.MaskFormatter;

import main.Controller;
import model.databaseModel.Fahrzeug;

/**
 * 
 * Fahrer-Klasse, um neue Fahrer ber das Men anzulegen.
 * 
 * @author $Author: Thomas Kramer
 * @version $Revision: $, $Date: 03.12.2012 UTC
 */
public class FahrerFrame extends JDialog implements Observer {
	private MainFrame mainFrame = null;

	private JFormattedTextField fkTextField = null;
	private JFormattedTextField fsTextField = null;

	// fr Comboboxen brauchen wir noch die IDs, nicht nur die Kennzeichen
	List<Fahrzeug> fahrzeugData = null;
	private JComboBox fg1CB = null;
	private JComboBox fg2CB = null;
	private JComboBox fg3CB = null;

	/**
	 * 
	 * Konstruktor
	 * 
	 */
	public FahrerFrame(MainFrame owner) {
		super(owner, "Anlegen eines neuen Fahrers");
		mainFrame = owner;

		setLayout(null);
		setSize(600, 300);

		/**
		 * Generieren aller Labels und Buttons
		 */
		JLabel dNLabel = new JLabel("Name des Fahrers");
		dNLabel.setBounds(20, 5, 200, 30);
		add(dNLabel);
		final JTextField dNTextField = new JTextField(0);
		dNTextField.setBounds(210, 5, 200, 30);
		add(dNTextField);

		MaskFormatter formatter;

		JLabel fkLabel = new JLabel("Fhrerscheinklasse");
		fkLabel.setBounds(20, 35, 200, 30);
		add(fkLabel);
		JLabel fsLabel = new JLabel("FhrerscheinSeit");
		fsLabel.setBounds(20, 65, 200, 30);
		add(fsLabel);

		/**
		 * Speziell formatierte Eingabefelder erzeugen
		 */
		try {
			// ? fr beliebige Zahl, # fr beliebiges Zeichen
			// siehe
			// http://www.tutego.de/blog/javainsel/2010/04/formatieren-mit-masken-der-maskformatter/
			formatter = new MaskFormatter("?#");
			formatter.setValidCharacters("ABCDE123");
			fkTextField = new JFormattedTextField(formatter);
			fkTextField.setBounds(210, 35, 200, 30);
			add(fkTextField);

			formatter = new MaskFormatter("##.##.####");
			formatter.setValidCharacters("0123456789");
			fsTextField = new JFormattedTextField(formatter);
			fsTextField.setBounds(210, 65, 200, 30);
			add(fsTextField);
		} catch (ParseException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		JLabel fg1 = new JLabel("1. Fahrzeug:");
		fg1.setBounds(20, 140, 200, 30);
		add(fg1);
		fg1CB = new JComboBox();
		fg1CB.setBounds(210, 140, 200, 30);
		add(fg1CB);
		JLabel fg2 = new JLabel("2. Fahrzeug:");
		fg2.setBounds(20, 170, 200, 30);
		add(fg2);
		fg2CB = new JComboBox();
		fg2CB.setBounds(210, 170, 200, 30);
		add(fg2CB);
		JLabel fg3 = new JLabel("3. Fahrzeug:");
		fg3.setBounds(20, 200, 200, 30);
		add(fg3);
		fg3CB = new JComboBox();
		fg3CB.setBounds(210, 200, 200, 30);
		add(fg3CB);

		JButton saveButton = new JButton("Speichern");
		saveButton.setBounds(470, 80, 100, 30);
		add(saveButton);
		saveButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						boolean failure = false;

						/**
						 * Fehlerberprfung
						 */
						failure = dNTextField.getText().trim().equals("");
						failure = failure
								|| (((String) fkTextField.getText()).trim()
										.equals(("")));
						failure = failure
								|| (((String) fsTextField.getText()).trim()
										.equals(("")));
						failure = failure
								|| (((String) fg1CB.getSelectedItem()).trim()
										.equals(("")));
						failure = failure
								|| (((String) fg1CB.getSelectedItem())
										.equals(((String) fg2CB
												.getSelectedItem())));
						failure = failure
								|| (((String) fg2CB.getSelectedItem())
										.equals(((String) fg3CB
												.getSelectedItem())) && !((String) fg2CB
										.getSelectedItem()).equals(""));

						/**
						 * wenn Fehler aufgetreten ist.... ansonsten speichern
						 */
						if (failure) {
							JOptionPane
									.showMessageDialog(
											null,
											"Es muss mindestens ein Fahrer mit seinen Fhrerscheindaten und das erste Fahrzeug eingegeben werden!",
											"Fehler", JOptionPane.ERROR_MESSAGE);
						} else {
							/**
							 * Speichern eines neuen Fahrers
							 */

							Controller.insertFahrerAndZustaendigkeit(
									dNTextField.getText().trim(), fkTextField
											.getText().trim(), fsTextField
											.getText().trim(), fahrzeugData
											.get(fg1CB.getSelectedIndex()),
									fahrzeugData.get(fg2CB.getSelectedIndex()),
									fahrzeugData.get(fg3CB.getSelectedIndex()));
							
							 /*
							  *  Erfolgsmeldung ausgeben
							 */
							JOptionPane
									.showMessageDialog(
											null,
											"Es wurde ein neuer Fahrer mitsamt Fahrzeugzuordnung(en) angelegt!",
											"Information",
											JOptionPane.INFORMATION_MESSAGE);

							/**
							 * Daten aktualisieren
							 */
							mainFrame.associations.doClick();
							dispose();
						}
					}
				});
			}
		});

		JButton closeButton = new JButton("Schlieen");
		closeButton.setBounds(470, 120, 100, 30);
		add(closeButton);
		closeButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						dispose();
					}
				});
			}
		});

		// Fllen der Comboboxen mit Daten
		update(null, null);

		// positionieren des Fensters: mittig
		setLocationRelativeTo(null);
	}

	/**
	 * Rckmeldung vom Observer-Pattern
	 */
	@Override
	public void update(Observable arg0, Object arg1) {
		/**
		 * Fllen der Comboboxen mit Daten
		 */
		fg1CB.removeAllItems();
		fg1CB.addItem("");
		fg2CB.removeAllItems();
		fg2CB.addItem("");
		fg3CB.removeAllItems();
		fg3CB.addItem("");
		fahrzeugData = Controller.selectFahrzeuge();
		for (Fahrzeug f : fahrzeugData) {
			fg1CB.addItem(f.getKennzeichen());
			fg2CB.addItem(f.getKennzeichen());
			fg3CB.addItem(f.getKennzeichen());
		}
		fg1CB.setSelectedIndex(0);
		fg2CB.setSelectedIndex(0);
		fg3CB.setSelectedIndex(0);
	}
}
