package view;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.util.List;
import java.util.Observable;
import java.util.Observer;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.WindowConstants;

import main.Controller;
import model.databaseModel.Fahrer;
import model.databaseModel.Fahrzeug;
import model.databaseModel.Zustaendigkeit;

/**
 * 
 * Die Hauptfenster-Klasse.
 * 
 * @author $Author: Thomas Kramer
 * @version $Revision: $, $Date: $ UTC
 */
@SuppressWarnings("serial")
public class MainFrame extends JFrame implements Observer {
	/**
	 * Unterfenster
	 */
	public FahrerFrame viewFF = null;

	private JScrollPane viewScrollPane = null;
	public JButton driver = null;
	public JButton cars = null;
	public JButton associations = null;
	final FahrerTable fahrerTable = new FahrerTable();
	final FahrzeugTable fahrzeugTable = new FahrzeugTable();
	final ZustaendigkeitTable zustaendigkeitTable = new ZustaendigkeitTable();

	/**
	 * Konstruktor.
	 */
	public MainFrame() {
		super("Aufgabe 8 fr Software-Engineering");

		// Make sure we have nice windows decorations.
		JFrame.setDefaultLookAndFeelDecorated(true);
		addWindowListener(new securityClosing());

		setLayout(new BorderLayout());
		final Container contentPane = getContentPane();
		contentPane.add(new MenuBar(this), BorderLayout.NORTH);

		final JPanel buttonPanel = new JPanel();
		buttonPanel.setLayout(new GridLayout(0, 3));
		driver = new JButton("Fahrer");
		cars = new JButton("Fahrzeuge");
		associations = new JButton("Zuordnungen");

		driver.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				if (viewScrollPane != null) {
					contentPane.remove(viewScrollPane);
				}
				viewScrollPane = new JScrollPane(fahrerTable);
				contentPane.add(viewScrollPane, BorderLayout.CENTER);
				update(null, null);

				// Aufruf der validate-Methode fr Neuzeichnen notwendig!
				contentPane.validate();
				contentPane.repaint();
			}
		});

		cars.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				if (viewScrollPane != null) {
					contentPane.remove(viewScrollPane);
				}
				viewScrollPane = new JScrollPane(fahrzeugTable);
				contentPane.add(viewScrollPane, BorderLayout.CENTER);
				update(null, null);

				// Aufruf der validate-Methode fr Neuzeichnen notwendig!
				contentPane.validate();
				contentPane.repaint();
			}
		});

		associations.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				if (viewScrollPane != null) {
					contentPane.remove(viewScrollPane);
				}
				viewScrollPane = new JScrollPane(zustaendigkeitTable);
				contentPane.add(viewScrollPane, BorderLayout.CENTER);
				update(null, null);

				// Aufruf der validate-Methode fr Neuzeichnen notwendig!
				contentPane.validate();
				contentPane.repaint();
			}
		});

		/**
		 * Buttons registrieren.
		 */
		buttonPanel.add(driver, BorderLayout.WEST);
		buttonPanel.add(cars, BorderLayout.WEST);
		buttonPanel.add(associations, BorderLayout.WEST);
		contentPane.add(buttonPanel, BorderLayout.SOUTH);

		/**
		 * Vorselektion.
		 */
		driver.doClick();
	}

	/**
	 * Benachrichtung vom Observer-Pattern
	 */
	@Override
	public void update(Observable arg0, Object arg1) {
		fahrerAnzeigen();
		fahrzeugeAnzeigen();
		zustaendigkeitAnzeigen();

		// Aufruf der validate-Methode fr Neuzeichnen notwendig!
		validate();
		repaint();
	}

	/**
	 * Alle Fahrer anzeigen
	 */
	private void fahrerAnzeigen() {
		/**
		 * Daten in View bernehmen
		 */
		fahrerTable.clearTable();
		List<Fahrer> fahrerData = Controller.selectFahrer();
		for (Fahrer f : fahrerData) {
			fahrerTable.dtm.addRow(new Object[] { f.getName(),
					f.getFuehrerscheinklasse(), f.getFuehrerscheinseit() });
		}
	}
	
	/**
	 * Alle Fahrzeuge anzeigen
	 */
	private void fahrzeugeAnzeigen() {
		/**
		 * Daten in View bernehmen
		 */		
		fahrzeugTable.clearTable();
		List<Fahrzeug> fahrzeugData = Controller.selectFahrzeuge();
		for (Fahrzeug f : fahrzeugData) {
			fahrzeugTable.dtm.addRow(new Object[] { f.getKennzeichen(),
					f.getErstzulassung() });
		}
	}	
	
	/**
	 * Alle Zustndigkeiten anzeigen
	 */
	private void zustaendigkeitAnzeigen() {
		/**
		 * Daten in View bernehmen
		 */				
		zustaendigkeitTable.clearTable();
		List<Zustaendigkeit> zustaendigkeitData = Controller
				.selectZustaendigkeit();
		for (Zustaendigkeit f : zustaendigkeitData) {
			zustaendigkeitTable.dtm.addRow(new Object[] {
					f.getFahrer().getName(), f.getFahrzeug1().getKennzeichen(),
					f.getFahrzeug2().getKennzeichen(),
					f.getFahrzeug3().getKennzeichen() });
		}
	}	

	/**
	 * 
	 * private innere Klasse, um Schlieen ber X-Button optional abzufangen und
	 * auf jeden Fall die Datenbank ber DataBaseHelper.shutdown() wieder
	 * herunterzufahren.
	 * 
	 * Den Shutdown-Prozess vor dem Schlieen automatisch aufzurufen (bei
	 * Benutzung des X-Buttons) funktionierte nicht wenn die MainFrame-Klasse
	 * direkt den WindowsListener implementiert - das windowClosing-Event dort
	 * wurde dann nicht aufgerufen obwohl es implementiert war.
	 * 
	 * So funktioniert es. Man beachte dass die Ausgabe "Datenbank-Shutdown" in
	 * jedem Fall erfolgt, egal wie die Anwendung geschlossen wird.
	 * 
	 * @author $Author: Thomas Kramer
	 * @version $Revision: $, $Date: $ UTC
	 */
	private class securityClosing extends WindowAdapter {
		public void windowClosing(WindowEvent e) {
			int option = JOptionPane.showOptionDialog(MainFrame.this,
					"Wollen Sie das Programm wirklich verlassen?",
					"Sicherheitsabfrage", JOptionPane.YES_NO_OPTION,
					JOptionPane.WARNING_MESSAGE, null, null, null);

			if (option == JOptionPane.YES_OPTION) {
				/*
				 * expliziter DB-Shutdown nicht mehr ntig, im Connection-String
				 * wurde festgelegt dass das DBMS automatisch runterfhrt sobald
				 * alle Verbindungen gelst wurden
				 */
				Controller.em.close();
				Controller.emf.close();
				System.exit(0);
			} else {
				// Abbruch
				setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
			}
		}
	}
}