package view;

import java.awt.Dialog.ModalityType;
import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JOptionPane;

import main.Controller;

/**
 * MenuBar-Klasse, die eine Titelleiste erstellt.
 * 
 * @author $Author: Thomas Kramer
 * @version $Revision: 1.0, $Date: 24.11.2012
 */

@SuppressWarnings("serial")
public class MenuBar extends JMenuBar {
	/**
	 * bergeordnetes Formular zwischenspeichern 
	 */
	private MainFrame mainFrame = null;
	
	/**
	 * Konstruktor.
	 */
	public MenuBar(MainFrame frame) {
		super();
		mainFrame = frame;

		add(new FileMenu(frame));
		add(new DriverMenu(frame));
		add(new HelpMenu(frame));
	}

	/**
	 * Die Dateimen-Klasse.
	 */
	private class FileMenu extends JMenu {
		/**
		 * 
		 * Konstruktor.
		 * 
		 * @param frame
		 *            Frame
		 */
		public FileMenu(final JFrame frame) {
			// Aufrufen des Basisklassenkonstruktors.
			super("Datei");

			// Anwendung beenden
			add(new AbstractAction("Beenden") {
				@Override
				public void actionPerformed(ActionEvent e) {
					/*
					 * expliziter DB-Shutdown nicht mehr ntig, im
					 * Connection-String wurde festgelegt dass das DBMS
					 * automatisch runterfhrt sobald alle Verbindungen gelst
					 * wurden
					 */
					Controller.em.close();					
					Controller.emf.close();					
					System.exit(0);
				}
			});
		}
	}

	/**
	 * Die Fahrermen-Klasse.
	 */
	private class DriverMenu extends JMenu {
		/**
		 * 
		 * Konstruktor.
		 * 
		 * @param frame
		 *            Frame
		 */
		public DriverMenu(final JFrame frame) {
			// Aufrufen des Basisklassenkonstruktors.
			super("Fahrer");

			// Anwendung beenden
			add(new AbstractAction("Anlegen") {
				@Override
				public void actionPerformed(ActionEvent e) {
					mainFrame.viewFF = new FahrerFrame(mainFrame);
					mainFrame.viewFF.setModalityType(ModalityType.DOCUMENT_MODAL);					
					mainFrame.viewFF.setVisible(true);
				}
			});
		}
	}

	/**
	 * Die Hilfsmen-Klasse
	 */
	private class HelpMenu extends JMenu {
		/**
		 * 
		 * Konstruktor.
		 * 
		 * @param frame
		 */
		public HelpMenu(final JFrame frame) {
			// Beschriftung
			super("Hilfe");

			// ffnen eines Infofensters.
			add(new AbstractAction("Info") {
				@Override
				public void actionPerformed(ActionEvent e) {
					JOptionPane.showMessageDialog(frame,
							"Eine Demo-Applikation fr HSQLDB.", "Info",
							JOptionPane.INFORMATION_MESSAGE);
				}
			});
		}
	}
}